@@ -300,6 +300,38 @@ pub fn HasSuffix(s: &str, suffix: &str) -> bool {
     s.ends_with(suffix)
 }
 
+/// Cut slices s around the first instance of sep,
+/// returning the text before and after sep.
+/// The found result reports whether sep appears in s.
+/// If sep does not appear in s, cut returns s, "", false.
+/// <details class="rustdoc-toggle top-doc">
+/// <summary class="docblock">zh-cn</summary>
+/// 在s中的第一个分隔字符串sep处切分字符串，返回sep前面部分before和sep后面的部分字符串after。found 值表示在s字符串中是否找到sep字符串。如果seq在s中找不到，切割结果返回(s,"",false)。
+/// </details>
+///
+/// # Example
+/// ```
+/// use gostd::strings;
+///
+/// assert_eq!(strings::Cut("127.0.0.1:8080",":"),("127.0.0.1","8080",true));
+/// ```
+pub fn Cut<'a>(s: &'a str, sep: &str) -> (&'a str, &'a str, bool) {
+    let before: &'a str;
+    let after: &'a str;
+    let found: bool;
+    let i = Index(s, sep);
+    if i >= 0 {
+        before = &s[..uint!(i)];
+        after = &s[(uint!(i) + len!(sep))..];
+        found = true;
+        return (before, after, found);
+    }
+    before = s;
+    after = "";
+    found = false;
+    (before, after, found)
+}
+
 /// Index returns the index of the first instance of substr in s, or -1 if substr is not present in s.
 /// <details class="rustdoc-toggle top-doc">
 /// <summary class="docblock">zh-cn</summary>
@@ -701,7 +733,7 @@ pub fn Replace<'a>(s: &'a str, old: &str, new: &str, n: int) -> String {
 /// assert_eq!("moo moo moo",strings::ReplaceAll("oink oink oink", "oink", "moo"));
 ///
 /// ```
-pub fn ReplaceAll<'a>(s: &'a str, old: &str, new: &str) -> String {
+pub fn ReplaceAll<'a>(s: &'a str, old: &'a str, new: &'a str) -> String {
     s.replace(old, new)
 }
 
@@ -1195,25 +1227,6 @@ impl Builder {
         String::from_utf8(self.buf.clone()).unwrap()
     }
 
-    /// Write appends the contents of p to b's buffer. Write always returns len!(p).
-    /// <details class="rustdoc-toggle top-doc">
-    /// <summary class="docblock">zh-cn</summary>
-    /// Write将p的内容附加到b的缓冲区, 写总是返回len!(p).
-    /// </details>
-    pub fn Write<'a>(&mut self, p: Vec<byte>) -> int {
-        self.buf.extend_from_slice(p.as_slice());
-        int!(len!(p))
-    }
-
-    /// WriteByte appends the byte c to b's buffer.
-    /// <details class="rustdoc-toggle top-doc">
-    /// <summary class="docblock">zh-cn</summary>
-    /// WriteByte将字节c追加到b的缓冲区。
-    /// </details>
-    pub fn WriteByte(&mut self, c: byte) {
-        self.buf.push(c)
-    }
-
     /// WriteRune appends the UTF-8 encoding of Unicode code point r to b's buffer. It returns the length of r.
     /// <details class="rustdoc-toggle top-doc">
     /// <summary class="docblock">zh-cn</summary>
@@ -1233,18 +1246,47 @@ impl Builder {
         self.buf = self.buf.get(..l + (n.abs() as uint)).unwrap().to_vec();
         return Ok(n);
     }
+}
 
+use crate::io::ByteWriter;
+impl ByteWriter for Builder {
+    /// WriteByte appends the byte c to b's buffer.
+    /// <details class="rustdoc-toggle top-doc">
+    /// <summary class="docblock">zh-cn</summary>
+    /// WriteByte将字节c追加到b的缓冲区。
+    /// </details>
+    fn WriteByte(&mut self, c: byte) -> Result<(), Error> {
+        self.buf.push(c);
+        Ok(())
+    }
+}
+
+use crate::io::StringWriter;
+impl StringWriter for Builder {
     /// WriteString appends the contents of s to b's buffer. It returns the length of s.
     /// <details class="rustdoc-toggle top-doc">
     /// <summary class="docblock">zh-cn</summary>
     /// WriteString将s的内容附加到b的缓冲区。它返回s的长度。
     /// </details>
-    pub fn WriteString(&mut self, s: &str) -> Result<int, &str> {
+    fn WriteString(&mut self, s: &str) -> Result<int, Error> {
         self.buf.append(s.as_bytes().to_vec().as_mut());
         return Ok(len!(self.buf) as int);
     }
 }
 
+use crate::io::Writer;
+impl Writer for Builder {
+    /// Write appends the contents of p to b's buffer. Write always returns len!(p).
+    /// <details class="rustdoc-toggle top-doc">
+    /// <summary class="docblock">zh-cn</summary>
+    /// Write将p的内容附加到b的缓冲区, 写总是返回len!(p).
+    /// </details>
+    fn Write<'a>(&mut self, p: Vec<byte>) -> Result<int, Error> {
+        self.buf.extend_from_slice(p.as_slice());
+        Ok(int!(len!(p)))
+    }
+}
+
 // Builder End
 /// A Reader implements the io.Reader, io.ReaderAt, io.ByteReader, io.ByteScanner,
 /// io.RuneReader, io.RuneScanner, io.Seeker, and io.WriterTo interfaces by reading
@@ -1254,6 +1296,7 @@ impl Builder {
 /// <summary class="docblock">zh-cn</summary>
 ///
 /// </details>
+#[derive(Default, PartialEq, PartialOrd, Debug, Clone)]
 pub struct Reader {
     s: String,
     i: int64,      // current reading index
@@ -1350,68 +1393,144 @@ impl io::ReaderAt for Reader {
 
 impl io::ByteReader for Reader {
     fn ReadByte(&mut self) -> Result<byte, Error> {
-        todo!()
+        self.prevRune = -1;
+        if self.i >= int64!(len!(self.s)) {
+            return Err(Error::new(ErrorKind::UnexpectedEof, "EOF"));
+        }
+        let b = self.s.as_bytes()[uint!(self.i)];
+        self.i += 1;
+        Ok(b)
     }
 }
 
 impl io::RuneReader for Reader {
     fn ReadRune(&mut self) -> Result<(rune, int), Error> {
-        todo!()
+        if self.i >= int64!(len!(self.s)) {
+            self.prevRune = -1;
+            return Err(Error::new(ErrorKind::UnexpectedEof, "EOF"));
+        }
+        self.prevRune = self.i as int;
+        if let Some(c) = self.s.chars().nth(uint!(self.i)) {
+            if rune!(c) < utf8::RuneSelf {
+                self.i += 1;
+                return Ok((rune!(c), 1));
+            }
+            let size = c.len_utf8() as int;
+            self.i += 1;
+            return Ok((c as rune, size as isize));
+        } else {
+            self.prevRune = -1;
+            return Err(Error::new(ErrorKind::UnexpectedEof, "EOF"));
+        }
     }
 }
 
+use io::Whence;
 impl io::Seeker for Reader {
-    fn Seek(&mut self, offset: int64, whence: int) -> Result<int64, Error> {
-        todo!()
+    fn Seek(&mut self, offset: int64, whence: Whence) -> int64 {
+        self.prevRune = -1;
+        let abs: int64;
+        match whence {
+            Whence::SeekStat => abs = offset,
+            Whence::SeekCurrent => abs = self.i + offset,
+            Whence::SeekEnd => abs = int64!(len!(self.s)) + offset,
+        }
+        self.i = abs;
+        abs
     }
 }
 
 impl io::ByteScanner for Reader {
-    fn UnreadByte(&self) -> Result<int, &str> {
-        todo!()
+    fn UnreadByte(&mut self) -> Result<(), Error> {
+        if self.i <= 0 {
+            return Err(Error::new(
+                ErrorKind::Other,
+                "strings.Reader.UnreadByte: at beginning of string",
+            ));
+        }
+        self.prevRune = -1;
+        self.i -= 1;
+        Ok(())
     }
 }
 
 impl io::WriterTo for Reader {
-    fn WriteTo(&self, w: Box<dyn io::Writer>) -> Result<int64, &str> {
-        todo!()
+    fn WriteTo(&mut self, w: Box<dyn io::Writer>) -> Result<int64, Error> {
+        self.prevRune = -1;
+        if self.i >= int64!(len!(self.s)) {
+            return Ok(0);
+        }
+        let s = self.s.get(self.i as usize..).unwrap();
+        if let Ok(m) = io::WriteString(w, s) {
+            if m > int!(len!(s)) {
+                panic!("gostd::strings::Reader::WriteTo: invalid WriteString count")
+            }
+            if m != int!(len!(s)) {
+                return Err(Error::new(ErrorKind::Other, "short write"));
+            }
+            self.i += int64!(m);
+            let n = int64!(m);
+            return Ok(n);
+        } else {
+            return Err(Error::new(ErrorKind::Other, "short write"));
+        }
     }
 }
 
-trait replacer {
-    fn Replace(&self, s: &str) -> &str
-    where
-        Self: Sized;
-    fn WriteString(&self, w: Box<dyn io::Writer>, s: string) -> Result<int, &str>
-    where
-        Self: Sized;
-}
-
-use std::sync::Once;
-/// NewReplacer returns a new Replacer from a list of old, new string pairs. Replacements are performed in the order they appear in the target string, without overlapping matches. The old string comparisons are done in argument order.
-
-/// NewReplacer panics if given an odd number of arguments.
-struct Replacer<'a> {
-    once: Once, // guards buildOnce method
-    r: Box<dyn replacer>,
-    oldnew: Vec<&'a str>,
+/// Replacer replaces a list of strings with replacements. It is safe for concurrent use by multiple goroutines.
+///
+/// <details class="rustdoc-toggle top-doc">
+/// <summary class="docblock">zh-cn</summary>
+/// Replacer类型进行一系列字符串的替换。
+/// </details>
+pub struct Replacer<'a> {
+    oldnew: Vec<(&'a str, &'a str)>,
 }
 
 impl<'a> Replacer<'a> {
-    /// NewReplacer returns a new Replacer from a list of old, new string pairs. Replacements are performed in the order they appear in the target string, without overlapping matches. The old string comparisons are done in argument order.
+    /// new returns a new Replacer from a list of old, new string pairs. Replacements are performed in the order they appear in the target string, without overlapping matches. The old string comparisons are done in argument order.
+    /// <details class="rustdoc-toggle top-doc">
+    /// <summary class="docblock">zh-cn</summary>
+    /// 使用提供的多组old、new字符串对创建并返回一个*Replacer。替换是依次进行的，匹配时不会重叠。
+    /// </details>
     ///
-    /// NewReplacer panics if given an odd number of arguments.
-    pub fn NewReplacer(oldnew: &str) -> &Replacer {
-        todo!()
+    /// # Example
+    ///
+    /// ```
+    /// use gostd::strings;
+    ///
+    ///    let p = vec![("<", "&lt;"), (">", "&gt;")];
+    ///    let r = strings::Replacer::new(p);
+    ///    let s = r.Replace("This is <b>HTML</b>!");
+    ///    println!("{}", s);
+    ///
+    /// ```
+    /// # Output
+    ///
+    /// ```text
+    /// This is &lt;b&gt;HTML&lt;/b&gt;!
+    /// ```
+    pub fn new(pairs: Vec<(&'a str, &'a str)>) -> Replacer<'a> {
+        Replacer { oldnew: pairs }
     }
-
     /// Replace returns a copy of s with all replacements performed.
-    pub fn Replace(&self, s: &str) -> &str {
-        todo!()
+    /// <details class="rustdoc-toggle top-doc">
+    /// <summary class="docblock">zh-cn</summary>
+    /// Replace返回s的所有替换进行完后的拷贝.
+    /// </details>
+    pub fn Replace(self, s: &str) -> String {
+        let mut new_str = s.to_owned();
+        for pair in self.oldnew.clone() {
+            new_str = ReplaceAll(new_str.as_str(), pair.0, pair.1);
+        }
+        new_str
     }
-
     /// WriteString writes s to w with all replacements performed.
-    pub fn WriteString(&self, w: Box<dyn io::Writer>, s: &str) -> Result<int, &str> {
-        todo!()
+    /// <details class="rustdoc-toggle top-doc">
+    /// <summary class="docblock">zh-cn</summary>
+    /// WriteString向w中写入s的所有替换进行完后的拷贝
+    /// </details>
+    pub fn WriteString(&self, mut w: Box<dyn io::Writer>, s: &str) -> Result<int, Error> {
+        w.Write(s.as_bytes().to_vec())
     }
 }
